class GamePlaying implements GameState {
  Backdrop bg;
  Robot rob;
  
  GameObjectCollection boots;
  GameObjectCollection enemies;
  GameObjectCollection explosions;
  
  boolean finale;
  
  int timeCounter;
  int wallTimer;
  int hunterTimer;
  
  GamePlaying() {
  }
  
  void init() {
    bg = new Backdrop();
    rob = new Robot(width/2, 350);
    boots = new GameObjectCollection();
    enemies = new GameObjectCollection();
    explosions = new GameObjectCollection();
    
    finale = false;
    timeCounter = 0;
    wallTimer = 0;
    hunterTimer = 60;
  }
  
  void step() {
    // input
    if (slashDetector.getSlash() && !finale) {
      rob.setArmAngle(slashDetector.getAngle());
      Boot b = new Boot(width/2, 350, slashDetector.getAngle());
      boots.add(b);
      shot.trigger();
    }
    // simulation
    spawnWall();
    spawnHunter();
    
    // our game will last around 30 seconds (600 updates at 20 updates/sec)
    timeCounter++;
    if (timeCounter > 600 && !finale) {
      enemies.add(new Train(790, 326, this));
      finale = true;
    }
    bg.step();
    rob.step();
    boots.stepAll();
    enemies.stepAll();
    explosions.stepAll();
    // collision
    boots.collideAll(enemies);
    enemies.collideAll(rob);
  }
  
  void render() {
    // drawing
    background(0);  
    bg.render();
    enemies.renderAll();
    rob.renderBody();
    boots.renderAll();
    rob.renderArm();
    explosions.renderAll();
    if (timeCounter > 660) {
      showFinaleText();
    }
  }
  
  void spawnWall() {
    if (wallTimer == 0) {
      if (!finale && random(1000) > 970) {
        enemies.add(new Wall(790, 350, this));
        wallTimer = 60;
      }
    }
    else
      wallTimer--;

  }

  void spawnHunter() {
    if (hunterTimer == 0) {
      if (!finale && random(1000) > 900) {
        float hx, hy;
        float k = random(900);
        if (k < 300) {
          hx = random(10, 790);
          hy = 0;
        }
        else if (k < 600) {
          hx = 790;
          hy = random(10, 590);
        }
        else {
          hx = 10;
          hy = random(10, 590);
        }
        float ang = atan2(rob.py - hy, rob.px - hx);
        enemies.add(new Hunter(int(hx), int(hy), ang, this));
        hunterTimer = 40;
      }
    }
    else
      hunterTimer--;

  }  
  
  void showFinaleText() {
    pushStyle();
    textFont(smallFont);
    textAlign(CENTER, TOP);
    noStroke();
    fill(255);
    text("Congratulations", width/2, 50);
    popStyle();
  }
}

