import processing.core.*; 
import processing.xml.*; 

import ddf.minim.*; 
import ddf.minim.signals.*; 
import ddf.minim.analysis.*; 
import ddf.minim.effects.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class spamboot extends PApplet {






/*
SPAM-BOOT-BOT
by Gilius (Ricardo Nakamura)
for the Impromptu Spambot Game Jam

Source code license: Creative Commons Attribution Share-Alike 3.0 (CC-BY-SA)

Uses the Arcade Classic font by Jakob Fischer / PizzaDude.
*/

// "Global" resources (yes, it's ugly but oh well)
// very large title font
PFont bigFont;
// medium font for messages
PFont smallFont;
// "gesture detector" LOL
SlashDetector slashDetector;
// audio library
Minim minim;
// the sound effects:
// shot
AudioSample shot;
// explosion
AudioSample boom;

// Collection of game states
GameState currState;
GameState[] states;
boolean changeState;
int nextState;

public void setup() {
  size(800, 600);
  frameRate(20);
 
  bigFont = loadFont("ArcadeClassic-200.vlw");
  smallFont = loadFont("ArcadeClassic-32.vlw");
  slashDetector = new SlashDetector();
  
  minim = new Minim(this);
  shot = minim.loadSample("shot.wav");
  boom = minim.loadSample("boom.wav");
  
  states = new GameState[3];
  states[0] = new GameTitle();
  states[1] = new GamePlaying();
  states[2] = new GameOver();
  currState = states[0];
  nextState = -1;
  changeState = false;
}

public void draw() {
  // Simulation
  currState.step();
  // Rendering
  currState.render();
  // Check if we should switch states
  if (changeState) {
    currState = states[nextState];
    currState.init();
    nextState = -1;
    changeState = false;
  }
}

public void stop() {
  shot.close();
  boom.close();
  minim.stop();
  super.stop();
}

public void mousePressed() {
  slashDetector.press(mouseX, mouseY);
}

public void mouseReleased() {
  slashDetector.release(mouseX, mouseY);
}

// Auxiliary methods 

public void beginGame() {
  nextState = 1;
  changeState = true;
}

public void showTitle() {
  nextState = 0;
  changeState = true;
}

public void gameOver() {
  nextState = 2;
  changeState = true;
}

class Backdrop {
  float baseA, baseB;
  float dxA, dxB;
  int nslices;

  Backdrop() {
    // magic numbers for the Perlin noise generator
    baseA = 42;
    baseB = 1004;
    dxA = 0;
    dxB = 0;
    nslices = 1 + width/10;
  }

  public void step() {
    // todo: make a more generic version with N parallax layers
    dxA+=0.25f;
    if (dxA > 10) {
      dxA = 0;
      baseA += 0.05f;
    }
    dxB+=1;
    if (dxB > 10) {
      dxB = 0;
      baseB += 0.1f;
    }
  }

  public void render() {
    pushMatrix();
    pushStyle();
    noStroke();
    fill(57, 13, 39);
    int x = 0;
    float p = baseA;
    translate(-dxA, 0);
    for (int i = 0; i < nslices; i++) {
      int h = PApplet.parseInt(noise(p)*100+100);
      rect(x, 400-h, 10, h);
      p += 0.05f;
      x += 10;
    }
    translate(dxA-dxB, 0);
    fill(95, 14, 62);
    x = 0;
    p = baseB;
    for (int i = 0; i < nslices; i++) {
      int h = PApplet.parseInt(noise(p)*50+50);
      rect(x, 400-h, 10, h);
      p += 0.1f;
      x += 10;
    }
    translate(dxB, 0);
    fill(142, 20, 79);
    rect(0, 400, width, height);
    popStyle();
    popMatrix();
  }
}

class Boot extends GameObject {
  float px, py, v;
  float rot;
  
  Boot(int x, int y, float r) {
    super();
    px = x;
    py = y;
    v = 10;
    rot = r;
  }
  
  public void step() {
    px += v*cos(rot);
    py += v*sin(rot);
    dead = (px < 0 || px > width || py < 0 || py > height);
  }
  
  public void render() {
    pushMatrix();
    pushStyle();
    translate(px, py);
    rotate(rot-PI/2);
    stroke(121,62,38);
    fill(188, 120, 57);
    beginShape();
      vertex(-10, -10);
      vertex(10, -10);
      vertex(10, 20);
      vertex(30, 30);
      vertex(-10, 30);
      vertex(-10, -10);
    endShape();
    fill(77, 41, 7);
    beginShape();
      vertex(30, 30);
      vertex(30, 33);
      vertex(5, 33);
      vertex(0, 30);
      vertex(0, 33);
      vertex(-10, 33);
      vertex(-10, 30);
      vertex(30, 30);
    endShape();
    popStyle();
    popMatrix();
  }
  
  public void testCollision(GameObject other) {
    other.testCollision(this);
  }
}

class Explosion extends GameObject {
  int px, py, sz, life, num;
  
  Explosion(int x, int y, int r, int n, int l) {
    super();
    px = x;
    py = y;
    sz = r;
    num = n;
    life = l;
  }
  
  public void step() {
    life--;
    dead = (life <= 0);
  }
  
  public void render() {
    pushMatrix();
    pushStyle();
    noStroke();
    translate(px, py);
    for (int i = 0; i < num; i++) {
      float c = random(200, 250);
      fill(c, c-20, 0);
      float r = random(40, 70);
      ellipse(random(-sz, sz), random(-sz, sz), r, r);
    }
    popStyle();
    popMatrix();
  }
  
  public void testCollision(GameObject other) {
  }
}

abstract class GameObject {
  boolean dead;
  
  GameObject() {
    dead = false;
  }
  
  public boolean isDead() {
    return dead;
  }

  public abstract void step();
  
  public abstract void render();
  
  public abstract void testCollision(GameObject other);
  public void testCollision(Boot other) {}
  public void testCollision(Hunter other) {}
  public void testCollision(Robot other) {}
  public void testCollision(Train other) {}
  public void testCollision(Wall other) {}
}

class GameObjectCollection {
  ArrayList col;
  
  GameObjectCollection() {
    col = new ArrayList();
  }
  
  public void add(GameObject x) {
    col.add(x);
  }
  
  public void stepAll() {
    for (int i = col.size()-1; i >= 0; i--) {
      GameObject g = (GameObject)col.get(i);
      g.step();
      if (g.isDead())
        col.remove(i);
    }
  }
  
  public void renderAll() {
    for (int i = 0; i < col.size(); i++) {
      GameObject g = (GameObject)col.get(i);
      g.render();
    }
  }
  
  public void collideAll(GameObjectCollection t) {
    for (int i = col.size()-1; i >= 0; i--) {
      GameObject g = (GameObject)col.get(i);
      t.collideAll(g);
      if (g.isDead())
        col.remove(i);
    }
  }

  // using "double dispatch" to handle collisions
  // albeit in a hackeneyed fashion  
  public void collideAll(GameObject w) {
    for (int i = col.size()-1; i >= 0; i--) {
      GameObject g = (GameObject)col.get(i);
      g.testCollision(w);
      w.testCollision(g);
      if (g.isDead())
        col.remove(i);
      if (w.isDead())
        break;
    }
  }
}

class GameOver implements GameState {
  public void init() {
  }
  
  public void step() {
    if (slashDetector.getSlash()) {
      showTitle();
    }
  }
  
  public void render() {
    pushStyle();
    textAlign(LEFT, TOP);
    background(57, 13, 39);
    noStroke();
    fill(255, 240, 0);
    textFont(bigFont);
    text("GAME", 10, 10);
    text("OVER", 10, 120);
    fill(255, 255, 0);
    textFont(smallFont);
    text("CLICK AND DRAG TO RETURN TO TITLE", 10, 470);
    popStyle();
  }
}

class GamePlaying implements GameState {
  Backdrop bg;
  Robot rob;
  
  GameObjectCollection boots;
  GameObjectCollection enemies;
  GameObjectCollection explosions;
  
  boolean finale;
  
  int timeCounter;
  int wallTimer;
  int hunterTimer;
  
  GamePlaying() {
  }
  
  public void init() {
    bg = new Backdrop();
    rob = new Robot(width/2, 350);
    boots = new GameObjectCollection();
    enemies = new GameObjectCollection();
    explosions = new GameObjectCollection();
    
    finale = false;
    timeCounter = 0;
    wallTimer = 0;
    hunterTimer = 60;
  }
  
  public void step() {
    // input
    if (slashDetector.getSlash() && !finale) {
      rob.setArmAngle(slashDetector.getAngle());
      Boot b = new Boot(width/2, 350, slashDetector.getAngle());
      boots.add(b);
      shot.trigger();
    }
    // simulation
    spawnWall();
    spawnHunter();
    
    // our game will last around 30 seconds (600 updates at 20 updates/sec)
    timeCounter++;
    if (timeCounter > 600 && !finale) {
      enemies.add(new Train(790, 326, this));
      finale = true;
    }
    bg.step();
    rob.step();
    boots.stepAll();
    enemies.stepAll();
    explosions.stepAll();
    // collision
    boots.collideAll(enemies);
    enemies.collideAll(rob);
  }
  
  public void render() {
    // drawing
    background(0);  
    bg.render();
    enemies.renderAll();
    rob.renderBody();
    boots.renderAll();
    rob.renderArm();
    explosions.renderAll();
    if (timeCounter > 660) {
      showFinaleText();
    }
  }
  
  public void spawnWall() {
    if (wallTimer == 0) {
      if (!finale && random(1000) > 970) {
        enemies.add(new Wall(790, 350, this));
        wallTimer = 60;
      }
    }
    else
      wallTimer--;

  }

  public void spawnHunter() {
    if (hunterTimer == 0) {
      if (!finale && random(1000) > 900) {
        float hx, hy;
        float k = random(900);
        if (k < 300) {
          hx = random(10, 790);
          hy = 0;
        }
        else if (k < 600) {
          hx = 790;
          hy = random(10, 590);
        }
        else {
          hx = 10;
          hy = random(10, 590);
        }
        float ang = atan2(rob.py - hy, rob.px - hx);
        enemies.add(new Hunter(PApplet.parseInt(hx), PApplet.parseInt(hy), ang, this));
        hunterTimer = 40;
      }
    }
    else
      hunterTimer--;

  }  
  
  public void showFinaleText() {
    pushStyle();
    textFont(smallFont);
    textAlign(CENTER, TOP);
    noStroke();
    fill(255);
    text("Congratulations", width/2, 50);
    popStyle();
  }
}

interface GameState {
  public void init();
  public void step();
  public void render();
}

class GameTitle implements GameState {
  public void init() {
  }
  
  public void step() {
    if (slashDetector.getSlash()) {
      beginGame();
    }
  }
  
  public void render() {
    pushStyle();
    textAlign(LEFT, TOP);
    background(57, 13, 39);
    noStroke();
    fill(255, 240, 0);
    textFont(bigFont);
    text("SPAM", 10, 10);
    text("BOOT", 10, 120);
    text("BOT", 10, 230);
    fill(255, 255, 0);
    textFont(smallFont);
    text("CLICK AND DRAG TO SHOOT", 10, 450);
    text("CLICK AND DRAG TO START GAME", 10, 470);
    popStyle();
  }
}

class Hunter extends GameObject {
  int px, py, v;
  float rot;  
  
  int life;
  GamePlaying parent;

  Hunter(int x, int y, float r, GamePlaying p) {
    px = x;
    py = y;
    rot = r;
    v = 7;
    life = 2;
    parent = p;
  }
  
  public void step() {
    px += v*cos(rot);
    py += v*sin(rot);
    dead = (px < 0 || px > width || py < 0 || py > height);
  }
  
  public void render() {
    pushMatrix();
    pushStyle();
    translate(px, py);
    strokeWeight(2);
    stroke(0, 0, 192);
    fill(0, 0, 255);
    ellipse(0, 0, 50, 50);
    rect(-25, -15, 50, 5);
    rect(-25, 10, 50, 5);
    fill(224, 224, 224);
    ellipse(0, 0, 20, 20);
    popStyle();
    popMatrix();
  }
  
  public void testCollision(GameObject other) {
    other.testCollision(this);
  }
  
  public void testCollision(Boot other) {
    if (dist(other.px, other.py, px, py) < 40) {
      other.dead = true;
      parent.explosions.add(new Explosion(PApplet.parseInt(other.px), PApplet.parseInt(other.py), 20, 3, 5));
      life--;
      if (life <= 0) {
        dead = true;
        parent.explosions.add(new Explosion(px, py, 30, 6, 20));
      }
      boom.trigger();
    }
  }
}

class Robot extends GameObject {
  int px, py, vx, vy;
  float r;
  float rlx, llx, lv;
  float rr;
  
  boolean bounce;
  
  
  Robot(int x, int y) {
    super();
    px = x;
    py = y;
    r = 0;
    rlx = -10;
    llx = 10;
    lv = 2;
    bounce = false;
    rr = 0;
  }

  public void getKicked() {
    bounce = true;
    vx = -12;
    vy = -20;
  }
  
  public void step() {
    rlx += lv;
    llx -= lv;
    if (rlx <= -10 || rlx >= 10)
      lv = -lv;
    if (bounce) {
      px += vx;
      py += vy;
      vy += 1;
      rr += 0.1f;
    }
  }

  public void render() {
  }

  public void renderBody() {
    pushMatrix();
    pushStyle();
    translate(px, py);
    rotate(rr);
    stroke(201, 201, 211);
    strokeWeight(2);
    pushMatrix();
    translate(llx, 40);
    fill(105, 105, 111);
    rect(-10, -5, 20, 30);
    popMatrix();
    pushMatrix();
    translate(rlx, 40);
    fill(118, 119, 129);
    rect(-10, -5, 20, 30);
    popMatrix();
    rect(-30, -20, 60, 60);
    rect(-15, -35, 30, 15);
    noStroke();
    fill(224, 0, 0);
    rect(-12, -41, 24, 5);
    stroke(201, 201, 211);
    fill(118, 119, 129);
    rect(-15, -50, 30, 10);
    popStyle();
    popMatrix();
  }
  
  public void renderArm() {
    pushMatrix();
    pushStyle();
    translate(px, py);
    rotate(r);
    stroke(201, 201, 211);
    fill(118, 119, 129);
    strokeWeight(2);
    rect(-5, -12, 33, 24);
    ellipse(0, 0, 26, 26);
    popStyle();
    popMatrix();
  }
  
  public void setArmAngle(float a) {
    r = a;
  }    
  
  public void testCollision(GameObject other) {
    other.testCollision(this);
  }
  
  public void testCollision(Wall other) {
    if (dist(other.px, other.py, px, py) < 40) {
      gameOver();
    }
  }
  
  public void testCollision(Hunter other) {
    if (dist(other.px, other.py, px, py) < 40) {
      gameOver();
    }
  }
}

class SlashDetector {
  int sX, sY, eX, eY;
  boolean slash;
  
  SlashDetector() {
    slash = false;
  }
  
  public void press(int x, int y) {
    sX = x;
    sY = y;
    slash = false;
  }
  
  public void release(int x, int y) {
    eX = x;
    eY = y;
    
    if (dist(sX, sY, eX, eY) > 50) {
      slash = true;
    }
  }

  public boolean getSlash() {
    boolean r = slash;
    slash = false;
    return r;
  }
  
  public int getStartX() {
    return sX;
  }

  public int getStartY() {
    return sY;
  }  
  
  public float getAngle() {
    return atan2(eY-sY, eX-sX);
  }
}

class Train extends GameObject {
  float ang;
  int px, py;
  GamePlaying parent;
  
  Train(int x, int y, GamePlaying p) {
    super();
    px = x;
    py = y;
    ang = 0;
    parent = p;
  }
  
  public void step() {
    px = px - 20;
    ang -= 0.4f;
    if (ang < -2*PI)
      ang += 2*PI;
  }
  
  public void render() {
    pushMatrix();
    pushStyle();
    stroke(0);
    fill(192);
    strokeWeight(3);
    translate(px, py);
    rect(-40, -50, 200, 100);
    rect(160, -80, 100, 130);
    triangle(-40, 20, -40, 90, -100, 90);
    pushMatrix();
    translate(0, 50);
    rotate(ang);
    ellipse(0, 0, 80, 80);
    rect(-5, -40, 10, 80);
    popMatrix();
    pushMatrix();
    translate(100, 50);
    rotate(ang+1);
    ellipse(0, 0, 80, 80);
    rect(-5, -40, 10, 80);
    popMatrix();  
    pushMatrix();
    translate(200, 40);
    rotate(ang+2);
    ellipse(0, 0, 100, 100);
    rect(-5, -50, 10, 100);
    popMatrix();
    popStyle();
    popMatrix();
  }
  
  public void testCollision(GameObject other) {
    other.testCollision(this);
  }

  public void testCollision(Robot other) {
    if (!other.isDead()) {
      if (px <= width/2 + 60) {
        other.dead = true;
        parent.explosions.add(new Explosion(width/2, 350, 60, 20, 15));
        other.getKicked();
      }
    }
  }
}

class Wall extends GameObject {
  int px, py;
  int life;
  GamePlaying parent;

  Wall(int x, int y, GamePlaying p) {
    px = x;
    py = y;
    life = 3;
    parent = p;
  }

  public void step() {
    px -= 2;
  }

  public void render() {
    pushMatrix();
    pushStyle();
    translate(px, py);
    strokeWeight(3);
    stroke(0, 64, 32);
    fill(0, 128, 64);
    rect(-20, -65, 40, 130);
    popStyle();
    popMatrix();
  }

  public void testCollision(GameObject other) {
    other.testCollision(this);
  }
  
  public void testCollision(Boot other) {
    if ((abs(other.px - px) < 50) && (other.py >= py - 60) && (other.py <= py + 60)) {
      other.dead = true;
      parent.explosions.add(new Explosion(PApplet.parseInt(other.px), PApplet.parseInt(other.py), 20, 3, 5));
      life--;
      if (life <= 0) {
        dead = true;
        parent.explosions.add(new Explosion(px, py, 50, 10, 20));
      }
      boom.trigger();
    }
  }
}

  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#ECE9D8", "spamboot" });
  }
}
