import ddf.minim.*;
import ddf.minim.signals.*;
import ddf.minim.analysis.*;
import ddf.minim.effects.*;

/*
SPAM-BOOT-BOT
by Gilius (Ricardo Nakamura)
for the Impromptu Spambot Game Jam

Source code license: Creative Commons Attribution Share-Alike 3.0 (CC-BY-SA)

Uses the Arcade Classic font by Jakob Fischer / PizzaDude.
*/

// "Global" resources (yes, it's ugly but oh well)
// very large title font
PFont bigFont;
// medium font for messages
PFont smallFont;
// "gesture detector" LOL
SlashDetector slashDetector;
// audio library
Minim minim;
// the sound effects:
// shot
AudioSample shot;
// explosion
AudioSample boom;

// Collection of game states
GameState currState;
GameState[] states;
boolean changeState;
int nextState;

void setup() {
  size(800, 600);
  frameRate(20);
 
  bigFont = loadFont("ArcadeClassic-200.vlw");
  smallFont = loadFont("ArcadeClassic-32.vlw");
  slashDetector = new SlashDetector();
  
  minim = new Minim(this);
  shot = minim.loadSample("shot.wav");
  boom = minim.loadSample("boom.wav");
  
  states = new GameState[3];
  states[0] = new GameTitle();
  states[1] = new GamePlaying();
  states[2] = new GameOver();
  currState = states[0];
  nextState = -1;
  changeState = false;
}

void draw() {
  // Simulation
  currState.step();
  // Rendering
  currState.render();
  // Check if we should switch states
  if (changeState) {
    currState = states[nextState];
    currState.init();
    nextState = -1;
    changeState = false;
  }
}

void stop() {
  shot.close();
  boom.close();
  minim.stop();
  super.stop();
}

void mousePressed() {
  slashDetector.press(mouseX, mouseY);
}

void mouseReleased() {
  slashDetector.release(mouseX, mouseY);
}

// Auxiliary methods 

void beginGame() {
  nextState = 1;
  changeState = true;
}

void showTitle() {
  nextState = 0;
  changeState = true;
}

void gameOver() {
  nextState = 2;
  changeState = true;
}

